package com.wanji.indicators.task.citybrainreceive.lanestatistics;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wanji.indicators.constant.Constant;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDataVO;
import com.wanji.indicators.task.citybrain.vo.LaneStatisticsDetail;
import com.wanji.indicators.task.citybrainreceive.CommonReceiveModel;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.po.CrossTurnDataRealtimePO;
import org.apache.flink.api.common.functions.MapFunction;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author duanruiming
 * @date 2024/04/16 11:06
 */
@Service
@Slf4j
public class LaneStatisticsReceive2TurnDataMap implements MapFunction<String, String> {
    @Override
    public String map(String value) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CommonReceiveModel model = mapper.readValue(value, CommonReceiveModel.class);
            List<CrossTurnDataRealtimePO> results = new ArrayList<>();
            String headers = model.getHeaders();
            String body = model.getBody();
            if (StringUtils.isNotBlank(headers) && (headers.contains(Constant.WAN_JI_NAME) || StringUtils.isBlank(body))) {
                return null;
            }
            List<LaneStatisticsDataVO> laneStatisticsDataVOS = mapper.readValue(body, new TypeReference<List<LaneStatisticsDataVO>>() {
            });
            if (!CollectionUtils.isEmpty(laneStatisticsDataVOS)) {
                for (LaneStatisticsDataVO inVo : laneStatisticsDataVOS) {
                    String crossId = inVo.getDeviceNo();
                    String startTime = inVo.getData_time();
                    Date dateTime = new Date(Long.parseLong(startTime));
                    String dir = inVo.getSectionNo();
                    List<LaneStatisticsDetail> dataList = inVo.getData_list();
                    if (!CollectionUtils.isEmpty(dataList)) {
                        for (LaneStatisticsDetail detail : dataList) {
                            String laneNo = detail.getLaneNo();
                            CrossTurnDataRealtimePO po = new CrossTurnDataRealtimePO();
                            po.setId(String.join(Constant.MIDDLE_LINE, crossId, dir, laneNo));
                            po.setCrossId(crossId);
                            po.setStartTime(dateTime);
                            po.setFlow(Integer.parseInt(detail.getWArrivalFlow()));
                            po.setSpeed(Double.parseDouble(detail.getSpeedAvg()));
                            po.setInSpeed(Double.parseDouble(detail.getSpeedAvg()));
                            po.setOutSpeed(Double.parseDouble(detail.getSpeedAvg()));
                            po.setQueueLength(Double.parseDouble(detail.getByQueueLen()));
                            po.setStopTimes(Double.parseDouble(detail.getByStoppingTimes()));
                            String wDelay = detail.getWDelay();
                            wDelay = StringUtils.isNotBlank(wDelay) ? wDelay : "0";
                            po.setDelayTime((int) Double.parseDouble(wDelay));
                            po.setVehheadDist(Double.parseDouble(detail.getGap()));
                            po.setVehheadTime(Double.parseDouble(detail.getHeadway()));
                            po.setQuality(0.6);
                            po.setNoStopRate(0.0);
                            po.setOneStopRate(0.0);
                            po.setTwoStopRate(0.0);
                            po.setThreeStopRate(0.0);
                            po.setSturation(0.7);
                            po.setBatchTime(Integer.parseInt(startTime.substring(0, 10)));
                            po.setEffusionRate(0.0);
                            po.setGreenLightEfficiency(0.0);
                            results.add(po);
                        }
                    }
                }
            }

            Map<String, List<CrossTurnDataRealtimePO>> map = new HashMap<>();
            map.put(Constant.REAL_TIME_TURN_DATA_KEY, results);
            return mapper.writeValueAsString(map);
        } catch (Exception e) {
            log.error("LaneStatisticsReceive2TurnDataMap:", e);
            throw new RuntimeException(e);
        }
    }
}